/* $Id: basic_cell.cpp,v 1.3 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "basic_cell.h"
#include "ICell.h"
#include "messenger.h"
#include "shape_sensor.h"
#include "shape_sensor_factory.h"
#include "Cell_data.h"
#include "utils.h"
#include "delivered_msg.h"
#include <vector>
#include <string>
using namespace std;


location::coord_t g_basic_cell_max_neighbor_dist;


//
// basic_cell
//

basic_cell::basic_cell(messenger *m,
							  Cell_data *cd,
							  const shape_sensor &ss,
							  const location &locn)
	: ICell(m, cd), ssens(ss), i_am_done(false),
	  steps(0),
	  my_locn(locn)
{}


location basic_cell::originating_locn(float signal_strength,
												  float signal_direction) const
{
	const float dist = signal_strength * g_basic_cell_max_neighbor_dist;
	const rel_location rel_locn = dist_angle2rellocn(dist, signal_direction);

	const location global_locn(rel_locn.get_x() + my_locn.get_x(),
										rel_locn.get_y() + my_locn.get_y());
	return global_locn;
}

void basic_cell::step()
{
	if(0==steps)
	{
		string text;
		if(ssens.on_shape())
			text += "Y";
		else
			text += "N";
		text += " (" + ftos(my_locn.get_x()) + "," + ftos(my_locn.get_y()) + ")";
		output(text);

		if(ssens.on_shape())
		{
			todeliver_msg m;
			auto_ptr<basic_msg> bm(new basic_msg());
			bm->text = ptos(reinterpret_cast<int*>(this));
			m.set_msg_content(static_cast< auto_ptr<msg_content> >(bm));
			send_msg(m);
		}
	}

	vector<delivered_msg> new_msgs = get_recved_msgs();
	if(new_msgs.size() > 0)
	{
		/*
		for(unsigned int i=0; i<new_msgs.size(); ++i)
		{
			string text = "[" + uitos(steps) + "] ";
			text += "M " + uitos(i+1) + "/" + uitos(new_msgs.size()) + " :";

			auto_ptr<msg_content> mc(new_msgs[i].extract_msg_content());
			auto_ptr<basic_msg>   bm(dynamic_cast<basic_msg*>(mc.release()));
			text += " from:" + bm->text;
			const location sender_locn
				= originating_locn(new_msgs[i].get_strength(),
										 new_msgs[i].get_direction());
			text += " (" + ftos(sender_locn.get_x())
				+ "," + ftos(sender_locn.get_y()) + ")";

			output(text);
		}
		*/
	}

	++steps;
	if(steps > 2000)
		i_am_done = true;
}

bool basic_cell::done() const
{
	return i_am_done;
}



//
// basic_cell_factory
//

basic_cell_factory::basic_cell_factory(const shape_sensor_factory &nssf,
													location::coord_t max_neighbor_dist)
	: ssf(nssf)
{
	g_basic_cell_max_neighbor_dist = max_neighbor_dist;
}

ICell* basic_cell_factory::operator()(Cell_data *cd, messenger *msger)
{
	assert(cd && msger);
	return new basic_cell(msger, cd, ssf(cd->get_location()), cd->get_location());
}



//
// basic_msg
//

basic_msg::basic_msg()
{}

basic_msg::basic_msg(const string &s) : text(s)
{}

basic_msg::~basic_msg()
{}

basic_msg* basic_msg::clone() const
{
	basic_msg *bm_copy = new basic_msg();
	bm_copy->text = text;
	return bm_copy;
}
