/* $Id: location.cpp,v 1.3 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "location.h"
#include <assert.h>
#include <iostream>
using std::ostream;

location::location()
	: locn_x(0),
	  locn_y(0)
{}

location::location(const location &locn)
	: locn_x(locn.get_x()),
	  locn_y(locn.get_y())
{}

location::location(coord_t x, coord_t y)
	: locn_x(x),
	  locn_y(y)
{
	assert(x >= 0);
	assert(y >= 0);
}


location::coord_t location::get_x() const
{ return locn_x; }

location::coord_t location::get_y() const
{ return locn_y; }


void location::set_x(location::coord_t x)
{ assert(x >= 0); locn_x = x; }

void location::set_y(location::coord_t y)
{ assert(y >= 0); locn_y = y; }


ostream& operator<<(ostream &os, const location& l)
{
	os << "(" << l.get_x() << "," << l.get_y() << ")";
	return os;
}



//
// rel_location
// TODO: make this and location use the same code
//

rel_location::rel_location()
	: locn_x(0),
	  locn_y(0)
{}

rel_location::rel_location(const rel_location &locn)
	: locn_x(locn.get_x()),
	  locn_y(locn.get_y())
{}

rel_location::rel_location(coord_t x, coord_t y)
	: locn_x(x),
	  locn_y(y)
{}


rel_location::coord_t rel_location::get_x() const
{ return locn_x; }

rel_location::coord_t rel_location::get_y() const
{ return locn_y; }


void rel_location::set_x(rel_location::coord_t x)
{ locn_x = x; }

void rel_location::set_y(rel_location::coord_t y)
{ locn_y = y; }


ostream& operator<<(ostream &os, const rel_location& rl)
{
	os << "{" << rl.get_x() << "," << rl.get_y() << "}";
	return os;
}



//
// utility functions
//
#include <math.h>

location::coord_t locn_distance(const location &locn1, const location &locn2)
{
	const location::coord_t delta_x = locn1.get_x() - locn2.get_x();
	const location::coord_t delta_y = locn1.get_y() - locn2.get_y();
	const location::coord_t dist    = sqrt(delta_x*delta_x + delta_y*delta_y);
	return dist;
}

float locn_direction(const location &from_locn, const location &to_locn)
{
	const location::coord_t delta_x = -from_locn.get_x() + to_locn.get_x();
	const location::coord_t delta_y = -from_locn.get_y() + to_locn.get_y();

	const float angle = atan2(delta_x, delta_y);

	return angle;
}

rel_location dist_angle2rellocn(float dist, float angle)
{
	const rel_location::coord_t rel_x(dist * sin(angle));
	const rel_location::coord_t rel_y(dist * cos(angle));

	const rel_location rel_locn(rel_x, rel_y);

	return rel_locn;
}
