/* $Id: location.h,v 1.3 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __LOCATION_H
#define __LOCATION_H

#include <iostream>

// The class location represents a location in space
class location
{
 public:
	typedef float coord_t;
	
	location();
	location(const location &locn);
	location(coord_t x, coord_t y);

	coord_t get_x() const;
	coord_t get_y() const;

	void set_x(coord_t x);
	void set_y(coord_t y);

 private:
	coord_t locn_x, locn_y;
};
std::ostream& operator<<(std::ostream& os, const location& l);

// This class represents a relative location, it allows negative coords
class rel_location
{
 public:
	typedef float coord_t;
	
	rel_location();
	rel_location(const rel_location &locn);
	rel_location(coord_t x, coord_t y);

	coord_t get_x() const;
	coord_t get_y() const;

	void set_x(coord_t x);
	void set_y(coord_t y);

 private:
	coord_t locn_x, locn_y;
};
std::ostream& operator<<(std::ostream& os, const rel_location& rl);


location::coord_t locn_distance( const location &locn1, const location &locn2);

// returns direction in radians from locn1 to locn2
float             locn_direction(const location &from_locn,
											const location &to_locn);

rel_location dist_angle2rellocn(float dist, float angle);

#endif // __LOCATION_H
