/* $Id: pbm_tester.cpp,v 1.2 2004/05/09 01:36:19 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <string>
#include <vector>
#include "pbm_toolkit.h"
using namespace std;

void display_image(unsigned int width, unsigned int height,
						 const vector< vector<bool> > &image);

int main(int argc, char **argv)
{
	if(2 != argc)
	{
		cerr << "Usage: pbm_tester <pbm_filename>" << endl;
		cerr << "About: pbm_tester loads pbm_filename and displays its magic"
			  << " number, size, and displays the image in ascii" << endl;
		exit(1);
	}

	const string filename(argv[1]);
	pbm p(filename);

	const string magic_num = p.get_magic_num();
	cout << "magic num: " << magic_num << endl;
	assert(magic_num == "P1");

	const unsigned int width = p.get_width();
	const unsigned int height = p.get_height();
	cout << "size: " << width << "x" << height << endl;

	cout << endl;
	cout << "Image:" << endl;
	display_image(width, height, p.get_image());

	cout << endl;
	const unsigned int scaled_width  = 25;
	const unsigned int scaled_height = 25;
	cout << "Scaled to 25x25:" << endl;
	display_image(scaled_width, scaled_height,
					  p.get_image(scaled_width, scaled_height));

	return 0;
}

void display_image(unsigned int width, unsigned int height,
						 const vector< vector<bool> > &image)
{
	for(unsigned int y=0; y<height; ++y)	
	{
		for(unsigned int x=0; x<width; ++x)
		{
			if(image[y][x]) cout << "X";
			else            cout << " ";
		}
		cout << endl;
	}
}
