/* $Id: rand_utils.h,v 1.5 2004/05/11 12:42:49 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __RAND_UTILS_H
#define __RAND_UTILS_H

#include <algorithm>


// About: A random-number-generating functor.
class rand_functor
{
 public:
	rand_functor(); // Seed with the current time
	rand_functor(unsigned int seed);

	unsigned int get_seed() const;
	void reseed(unsigned int seed);


	// Returns an int uniform rnd number in (0, N) using rand
	int operator()(int N);

	// Returns a double uniform rnd number in [0,N] using rand
	double operator()(double N);

 private:
	unsigned int rnd_seed;
};

rand_functor& global_randf();


// About: A replacement for the stl's random_sample()
// Why:
// random_sample() (when tested on this computer), when the input and output
// ranges were of equal size, did not shuffle the list.
// This function calls random_sample() unless the given ranges are equal,
// in which case randome_shuffle() is used.

template<class InputIterator,
			class RandAccessIterator,
			class RandomNumberGenerator>
void random__sample_shuffle(InputIterator first,
									 InputIterator last,
									 RandAccessIterator ofirst,
									 RandAccessIterator olast,
									 RandomNumberGenerator &rand,
									 bool in_out_ranges_equal)
{
	if(!in_out_ranges_equal)
	{
		random_sample(first, last, ofirst, olast, rand);
	}
	else
	{
		copy(first, last, ofirst);
		random_shuffle(ofirst, olast, rand);
	}
}

#endif // __RAND_UTILS_H
