/* $Id: shape_cell.h,v 1.4 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SHAPE_CELL_H
#define __SHAPE_CELL_H

#include "ICell.h"
#include "messenger.h"
#include "shape_sensor.h"
#include "shape_sensor_factory.h"
#include "delivered_msg.h"
#include "msg_content.h"
#include <vector>
class Cell_data;

// Data a cell has on the shape, information from a neighbor
class neighbor_shape_data
{
 public:
	neighbor_shape_data(unsigned int id, float d, float a, bool os);

	unsigned int id;
	float        distance;
	float        angle;
	bool         on_shape;
};


class shape_cell : public ICell
{
 public:
	shape_cell(messenger *m, Cell_data *cd, const shape_sensor &ss,
				  unsigned int id);

	// ICell interface
 public:
	virtual void step();
	virtual bool done() const;

	// Methods for inspecting this cell
 public:
	std::vector<neighbor_shape_data> get_shape_data()   const;
	bool                             on_shape()         const;
	unsigned int                     get_id()           const;

	// Only for looking at what actually happened
	float                            get_angle_offset() const;

 private:
	// override ICell::output to output our id rather than pointer
	virtual void output(const std::string &text);

	// fit msg direction and distance data to sensor sensitivity,
	// we do this in the cell because the simulator doesn't know about our
	// sensor's abilities
	void levelize_msg_metadata(std::vector<delivered_msg> &dms);

	void extract_and_process_descrip_msgs(std::vector<delivered_msg> &dms);


	bool               have_sampled_env;
	shape_sensor       ssens;
	const unsigned int id;
	unsigned int steps_since_last_msg;
	// the number of steps since the last msg till mark self as done
	static const unsigned int post_last_msg_steps_till_done = 20;
	// offset of the 0 radian mark, set to rnd value
	const float  angle_offset;

	std::vector<neighbor_shape_data> shape_data;
};

class shape_cell_factory
{
 public:
	shape_cell_factory(const shape_sensor_factory &ssf,
							 location::coord_t max_neighbor_dist,
							 int num_strength_levels,
							 int num_dir_levels);

	ICell* operator()(Cell_data *cd, messenger *msger);
							
 private:
	shape_sensor_factory ssf;
	unsigned int next_cell_id;
};


#include <string>
// descrip_msg is used to pass shape description information among cells
class descrip_msg : public msg_content
{
 public:
	descrip_msg();
	descrip_msg(unsigned int id, bool on_shape);
	virtual ~descrip_msg();
	virtual descrip_msg* clone() const;

	unsigned int id;
	bool on_shape;
};

// heard_all_neighbors_msg is used to inform neighbors a cell has heard from
// all its neighbors
class heard_all_neighbors_msg : public msg_content
{
 public:
	heard_all_neighbors_msg();
	virtual ~heard_all_neighbors_msg();
	virtual heard_all_neighbors_msg* clone() const;
};

#endif // __SHAPE_CELL_H
