/* $Id: shape_cell_querier.h,v 1.3 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SHAPE_CELL_QUERIER_H
#define __SHAPE_CELL_QUERIER_H

#include "shape_cell.h"
#include "location.h"
#include "cd_list.h"
#include <vector>
#include <functional>
#include <string>


// data read from a cell
class cell_data
{
 public:
	cell_data(unsigned int                            cell_id,
				 bool                                    cell_on_shape,
				 const std::vector<neighbor_shape_data> &nghbr_shape_data);

	unsigned int                     id;
	bool                             on_shape;
	std::vector<neighbor_shape_data> nghbrs;

};

class cell_data_comp : public std::binary_function<const cell_data &,
							                              const cell_data &,
							                              bool>
{ public: bool operator()(const cell_data &x, const cell_data &y); };


// data from the simulator for a cell
class sim_data
{
 public:
	sim_data(const location  &sim__cell_locn,
				float            sim__angle_offset,
				const cd_list_t &sim__ngbhrs);
	sim_data(const location                  &sim__cell_locn,
				float                            sim__angle_offset,
				const std::vector<unsigned int> &sim__ngbhr_ids);

	location                         locn;
	float                            angle_offset;
	std::vector<unsigned int>        nghbrs;
};

class shape_data
{
 public:
	shape_data(const cell_data &_cd, const sim_data &_sd);
	//	shape_data(const shape_data &_sd);

	cell_data cd;
	bool cd_valid;
	sim_data  sd;
};

class shape_data_comp : public std::binary_function<const shape_data &,
							                               const shape_data &,
							                               bool>
{ public: bool operator()(const shape_data &x, const shape_data &y); };



void query_cells(const cd_list_t::const_iterator cell_begin,
					  const cd_list_t::const_iterator cells_end,
					  vector<shape_data>             &shape_data_vec,
					  bool                            output_progress);

bool save_queried_data(const std::string             &filename,
							  const std::vector<shape_data> &shaped_vec,
							  const location                &space_size,
							  const location::coord_t        neighbor_max_dist,
							  const std::string             &locn_nghbr_filename,
							  const std::string             &shape_descrip_filename,
							  unsigned int                   prng_seed,
							  bool                           output_progress);

bool load_queried_data(const std::string       &filename,
							  std::vector<shape_data> &shaped_vec,
							  location                &space_size,
							  location::coord_t       &neighbor_max_dist,
							  std::string             &locn_nghbr_filename,
							  std::string             &shape_descrip_filename,
							  unsigned int             prng_seed,
							  bool                     output_progress);

#endif // __SHAPE_CELL_QUERIER_H
