/* $Id: shapsim.cpp,v 1.5 2004/05/09 01:36:20 chris Exp $
 *
 * shapsim - Amorphous shape mapping computer simulator and analyzer.
 * Copyright (C) 2004 Chris Frost <frost@cs.virginia.edu>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "simulator.h"
#include "shape_cell.h"
#include "location.h"
#include "location_neighbor_generator.h"
#include "shape_cell_querier.h"
#include <iostream>
#include <string>
#include <vector>
using namespace std;

int main(int argc, char **argv)
{
	const bool verbose_output    = true;
	const int  required_num_args = 6;

	if( !(required_num_args == argc || 1+required_num_args == argc) )
	{
		cerr << "Usage: shapsim <query_results_filename_prefix>"
			  << " <shape_description_filename>"
			  << " <locn_nghbr_filename> <num_strength_levels> <num_dir_levels>"
			  << " [prng_seed]"
			  << endl;
		exit(1);
	}

	if(1+required_num_args == argc)
		global_randf().reseed(atoi(argv[required_num_args]));

	cout << "Initializing simulator (prng_seed=" << global_randf().get_seed()
		  << ")..." << endl;

	const string      query_results_filename_prefix(argv[1]);
	const string      shape_description_filename(argv[2]);
	const string      locn_nghbr_filename(argv[3]);
	const int         num_strength_levels(atoi(argv[4]));
	const int         num_dir_levels(atoi(argv[5]));

	location::coord_t neighbor_max_dist;
	location          space_size;
	location_neighbor_generator lng;
	assert(lng.load__nbhr_dist__space_size(locn_nghbr_filename,
														space_size, neighbor_max_dist));

	const string init_indent = "   ";

	cout << init_indent << "shape sensor factory" << endl;
	shape_sensor_factory          ssf(space_size, shape_description_filename);

	cout << init_indent << "shape cell factory" << endl;
	shape_cell_factory            scf(ssf, neighbor_max_dist,
												 num_dir_levels, num_strength_levels);

	cout << init_indent << "simulator" << endl;
	simulator<shape_cell_factory> sim(locn_nghbr_filename,
												 space_size,
												 neighbor_max_dist,
												 scf);
	cout << "Simulator initialized, running simulation." << endl;

	sim.run();

	cout << "Cells have completed their shape sensing, querying cells." <<endl;
	vector<shape_data> sd;
	query_cells(sim.get_cells()->begin(), sim.get_cells()->end(), sd,
					verbose_output);

	cout << "Cells queried, saving query results." << endl;	
	assert(save_queried_data(query_results_filename_prefix+"-queried_data",
									 sd, space_size, neighbor_max_dist,
									 locn_nghbr_filename, shape_description_filename,
									 global_randf().get_seed(),
									 verbose_output));

	return 0;
}
